@@ -1,6 +1,7 @@
 /* -*- mode: C; c-basic-offset: 4 -*-
  * Drive Mount Applet
  * Copyright (c) 2004 Canonical Ltd
+ * Copyright 2008 Pierre Ossman
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -24,6 +25,7 @@
 #  include <config.h>
 #endif
 
+#include <gio/gio.h>
 #include "drive-button.h"
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -43,10 +45,10 @@ enum {
 /* type registration boilerplate code */
 G_DEFINE_TYPE(DriveButton, drive_button, GTK_TYPE_BUTTON)
 
-static void     drive_button_set_drive    (DriveButton    *self,
-				           GnomeVFSDrive  *drive);
 static void     drive_button_set_volume   (DriveButton    *self,
-				           GnomeVFSVolume *volume);
+				           GVolume        *volume);
+static void     drive_button_set_mount    (DriveButton    *self,
+				           GMount         *mount);
 static void     drive_button_reset_popup  (DriveButton    *self);
 static void     drive_button_ensure_popup (DriveButton    *self);
 
@@ -92,8 +94,8 @@ drive_button_init (DriveButton *self)
     gtk_container_add (GTK_CONTAINER (self), image);
     gtk_widget_show(image);
 
-    self->drive = NULL;
     self->volume = NULL;
+    self->mount = NULL;
     self->icon_size = 24;
     self->update_tag = 0;
 
@@ -101,12 +103,12 @@ drive_button_init (DriveButton *self)
 }
 
 GtkWidget *
-drive_button_new (GnomeVFSDrive *drive)
+drive_button_new (GVolume *volume)
 {
     DriveButton *self;
 
     self = g_object_new (DRIVE_TYPE_BUTTON, NULL);
-    drive_button_set_drive (self, drive);
+    drive_button_set_volume (self, volume);
     
     g_signal_connect (gtk_icon_theme_get_default (),
         "changed", G_CALLBACK (drive_button_theme_change),
@@ -116,12 +118,12 @@ drive_button_new (GnomeVFSDrive *drive)
 }
 
 GtkWidget *
-drive_button_new_from_volume (GnomeVFSVolume *volume)
+drive_button_new_from_mount (GMount *mount)
 {
     DriveButton *self;
 
     self = g_object_new (DRIVE_TYPE_BUTTON, NULL);
-    drive_button_set_volume (self, volume);
+    drive_button_set_mount (self, mount);
     
     g_signal_connect (gtk_icon_theme_get_default (),
         "changed", G_CALLBACK (drive_button_theme_change),
@@ -135,7 +137,7 @@ drive_button_destroy (GtkObject *object)
 {
     DriveButton *self = DRIVE_BUTTON (object);
 
-    drive_button_set_drive (self, NULL);
+    drive_button_set_volume (self, NULL);
 
     if (self->update_tag)
 	g_source_remove (self->update_tag);
@@ -261,41 +263,41 @@ drive_button_theme_change (GtkIconTheme *icon_theme, gpointer data)
 }
 
 static void
-drive_button_set_drive (DriveButton *self, GnomeVFSDrive *drive)
+drive_button_set_volume (DriveButton *self, GVolume *volume)
 {
     g_return_if_fail (DRIVE_IS_BUTTON (self));
 
-    if (self->drive) {
-	gnome_vfs_drive_unref (self->drive);
-    }
-    self->drive = NULL;
     if (self->volume) {
-	gnome_vfs_volume_unref (self->volume);
+	g_object_unref (self->volume);
     }
     self->volume = NULL;
+    if (self->mount) {
+	g_object_unref (self->mount);
+    }
+    self->mount = NULL;
 
-    if (drive) {
-	self->drive = gnome_vfs_drive_ref (drive);
+    if (volume) {
+	self->volume = g_object_ref (volume);
     }
     drive_button_queue_update (self);
 }
 
 static void
-drive_button_set_volume (DriveButton *self, GnomeVFSVolume *volume)
+drive_button_set_mount (DriveButton *self, GMount *mount)
 {
     g_return_if_fail (DRIVE_IS_BUTTON (self));
 
-    if (self->drive) {
-	gnome_vfs_drive_unref (self->drive);
-    }
-    self->drive = NULL;
     if (self->volume) {
-	gnome_vfs_volume_unref (self->volume);
+	g_object_unref (self->volume);
     }
     self->volume = NULL;
+    if (self->mount) {
+	g_object_unref (self->mount);
+    }
+    self->mount = NULL;
 
-    if (volume) {
-	self->volume = gnome_vfs_volume_ref (volume);
+    if (mount) {
+	self->mount = g_object_ref (mount);
     }
     drive_button_queue_update (self);
 }
@@ -306,10 +308,12 @@ drive_button_update (gpointer user_data)
     DriveButton *self;
     GdkScreen *screen;
     GtkIconTheme *icon_theme;
-    char *icon_name;
+    GtkIconInfo *icon_info;
+    GIcon *icon;
     int width, height;
     GdkPixbuf *pixbuf, *scaled;
     GtkRequisition button_req, image_req;
+    char *display_name, *tip;
 
     g_return_val_if_fail (DRIVE_IS_BUTTON (user_data), FALSE);
     self = DRIVE_BUTTON (user_data);
@@ -317,54 +321,41 @@ drive_button_update (gpointer user_data)
 
     drive_button_reset_popup (self);
 
-    /* if no drive or volume, unset image */
-    if (!self->drive && !self->volume) {
+    /* if no volume or mount, unset image */
+    if (!self->volume && !self->mount) {
 	if (GTK_BIN (self)->child != NULL)
 	    gtk_image_set_from_pixbuf (GTK_IMAGE (GTK_BIN (self)->child), NULL);
 	return FALSE;
     }
 
+    if (self->volume) {
+	GMount *mount;
 
-    char *display_name, *tip;
+	display_name = g_volume_get_name (self->volume);
+	mount = g_volume_get_mount (self->volume);
 
-    if (self->drive) {
-	display_name = gnome_vfs_drive_get_display_name (self->drive);
-	if (gnome_vfs_drive_is_mounted (self->drive))
+	if (mount)
 	    tip = g_strdup_printf ("%s\n%s", display_name, _("(mounted)"));
-	else if (gnome_vfs_drive_is_connected (self->drive))
-	    tip = g_strdup_printf ("%s\n%s", display_name, _("(not mounted)"));
 	else
-	    tip = g_strdup_printf ("%s\n%s", display_name, _("(not connected)"));
+	    tip = g_strdup_printf ("%s\n%s", display_name, _("(not mounted)"));
+
+	if (mount)
+	    icon = g_mount_get_icon (mount);
+	else
+	    icon = g_volume_get_icon (self->volume);
+
+	if (mount)
+	    g_object_unref (mount);
     } else {
-	display_name = gnome_vfs_volume_get_display_name (self->volume);
-	if (gnome_vfs_volume_is_mounted (self->volume))
-	    tip = g_strdup_printf ("%s\n%s", display_name, _("(mounted)"));
-	else
-	    tip = g_strdup_printf ("%s\n%s", display_name, _("(not mounted)"));
+	display_name = g_mount_get_name (self->mount);
+	tip = g_strdup_printf ("%s\n%s", display_name, _("(mounted)"));
+	icon = g_mount_get_icon (self->mount);
     }
 
     gtk_widget_set_tooltip_text (GTK_WIDGET (self), tip);
     g_free (tip);
     g_free (display_name);
 
-    /* get the icon for the first mounted volume, or the drive itself */
-    if (self->drive) {
-	if (gnome_vfs_drive_is_mounted (self->drive)) {
-	    GList *volumes;
-	    GnomeVFSVolume *volume;
-
-	    volumes = gnome_vfs_drive_get_mounted_volumes (self->drive);
-	    volume = GNOME_VFS_VOLUME (volumes->data);
-	    icon_name = gnome_vfs_volume_get_icon (volume);
-	    g_list_foreach (volumes, (GFunc)gnome_vfs_volume_unref, NULL);
-	    g_list_free (volumes);
-	} else {
-	    icon_name = gnome_vfs_drive_get_icon (self->drive);
-	}
-    } else {
-	icon_name = gnome_vfs_volume_get_icon (self->volume);
-    }
-
     /* base the icon size on the desired button size */
     gtk_widget_size_request (GTK_WIDGET (self), &button_req);
     gtk_widget_size_request (GTK_BIN (self)->child, &image_req);
@@ -373,9 +364,16 @@ drive_button_update (gpointer user_data)
 
     screen = gtk_widget_get_screen (GTK_WIDGET (self));
     icon_theme = gtk_icon_theme_get_for_screen (screen);
-    pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name,
-				       MIN (width, height), 0, NULL);
-    g_free (icon_name);
+    icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon,
+    						MIN (width, height),
+    						GTK_ICON_LOOKUP_USE_BUILTIN);
+    if (icon_info)
+    {
+	pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+	gtk_icon_info_free (icon_info);
+    }
+
+    g_object_unref (icon);
 
     if (!pixbuf)
 	return FALSE;
@@ -417,16 +415,38 @@ int
 drive_button_compare (DriveButton *button, DriveButton *other_button)
 {
     /* sort drives before driveless volumes volumes */
-    if (button->drive) {
-	if (other_button->drive)
-	    return gnome_vfs_drive_compare (button->drive, other_button->drive);
-	else
+    if (button->volume) {
+	if (other_button->volume)
+	{
+	    int cmp;
+	    gchar *str1, *str2;
+
+	    str1 = g_volume_get_name (button->volume);
+	    str2 = g_volume_get_name (other_button->volume);
+	    cmp = g_utf8_collate (str1, str2);
+	    g_free (str2);
+	    g_free (str1);
+
+	    return cmp;
+	} else {
 	    return -1;
+	}
     } else {
-	if (other_button->drive)
+	if (other_button->volume)
+	{
 	    return 1;
-	else
-	    return gnome_vfs_volume_compare (button->volume, other_button->volume);
+	} else {
+	    int cmp;
+	    gchar *str1, *str2;
+
+	    str1 = g_mount_get_name (button->mount);
+	    str2 = g_mount_get_name (other_button->mount);
+	    cmp = g_utf8_collate (str1, str2);
+	    g_free (str2);
+	    g_free (str1);
+
+	    return cmp;
+	}
     }
 }
 
@@ -469,15 +489,15 @@ escape_underscores (const char *str)
     return new_str;
 }
 static GtkWidget *
-create_menu_item (DriveButton *self, const gchar *stock_id,
+create_menu_item (DriveButton *self, const gchar *icon_name,
 		  const gchar *label, GCallback callback,
 		  gboolean sensitive)
 {
     GtkWidget *item, *image;
 
     item = gtk_image_menu_item_new_with_mnemonic (label);
-    if (stock_id) {
-	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
+    if (icon_name) {
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_widget_show (image);
     }
@@ -499,26 +519,27 @@ open_drive (DriveButton *self, GtkWidget *item)
 
     screen = gtk_widget_get_screen (GTK_WIDGET (self));
 
-    if (self->drive) {
-	argv[1] = gnome_vfs_drive_get_activation_uri (self->drive);
-	/* if the drive has no activation URI, get the activation URI
-	 * of it's first mounted volume */
-	if (!argv[1]) {
-	    GList *volumes;
-	    GnomeVFSVolume *volume;
+    if (self->volume) {
+	GMount *mount;
 
-	    volumes = gnome_vfs_drive_get_mounted_volumes (self->drive);
-	    if (volumes) {
-		volume = GNOME_VFS_VOLUME (volumes->data);
+	mount = g_volume_get_mount (self->volume);
+	if (mount) {
+	    GFile *file;
 
-		argv[1] = gnome_vfs_volume_get_activation_uri (volume);
-		g_list_foreach (volumes, (GFunc)gnome_vfs_volume_unref, NULL);
-		g_list_free (volumes);
-	    }
+	    file = g_mount_get_root (mount);
+
+	    argv[1] = g_file_get_uri (file);
+	    g_object_unref(file);
+
+	    g_object_unref(mount);
 	}
-    } else if (self->volume)
-	argv[1] = gnome_vfs_volume_get_activation_uri (self->volume);
-    else
+    } else if (self->mount) {
+	GFile *file;
+
+	file = g_mount_get_root (self->mount);
+	argv[1] = g_file_get_uri (file);
+	g_object_unref(file);
+    } else
 	g_return_if_reached();
 
     if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
@@ -530,7 +551,7 @@ open_drive (DriveButton *self, GtkWidget *item)
 						     GTK_BUTTONS_OK,
 						     _("Cannot execute '%s'"),
 						     argv[0]);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message, NULL);
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_object_destroy), NULL);
 	gtk_widget_show (dialog);
@@ -539,84 +560,6 @@ open_drive (DriveButton *self, GtkWidget *item)
     g_free (argv[1]);
 }
 
-static void
-mount_result (gboolean succeeded,
-	      char *error,
-	      char *detailed_error,
-	      gpointer data)
-{
-    GtkWidget *dialog, *hbox, *vbox, *image, *label;
-    char *title, *str;
-
-    if (!succeeded) {
-	switch (GPOINTER_TO_INT(data)) {
-	case CMD_MOUNT_OR_PLAY:
-	    title = _("Mount Error");
-	    break;
-	case CMD_UNMOUNT:
-	    title = _("Unmount Error");
-	    break;
-	case CMD_EJECT:
-	    title = _("Eject Error");
-	    break;
-	default:
-	    title = _("Error");
-	}
-
-	dialog = gtk_dialog_new ();
-	atk_object_set_role (gtk_widget_get_accessible (dialog), ATK_ROLE_ALERT);
-	gtk_window_set_title (GTK_WINDOW (dialog), title);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
-	hbox = gtk_hbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
-			    FALSE, FALSE, 0);
-
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR,
-					  GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-	gtk_widget_show (image);
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-
-	vbox = gtk_vbox_new (FALSE, 12);
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-	gtk_widget_show (vbox);
-
-	str = g_strconcat ("<span weight=\"bold\" size=\"larger\">",
-			   error, "</span>", NULL);
-	label = gtk_label_new (str);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-	g_free (str);
-
-	label = gtk_label_new (detailed_error);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK,
-			       GTK_RESPONSE_OK);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-					 GTK_RESPONSE_YES);
-
-	gtk_widget_show (dialog);
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (gtk_object_destroy), NULL);
-
-    }
-}
-
 /* copied from gnome-volume-manager/src/manager.c maybe there is a better way than
  * duplicating this code? */
 
@@ -701,28 +644,69 @@ gvm_check_dvd_only (const char *udi, const char *device, const char *mount_point
 /* END copied from gnome-volume-manager/src/manager.c */
 
 static gboolean
-check_dvd_video (GnomeVFSVolume *volume)
+check_dvd_video (DriveButton *self)
 {
-	char *device_path = gnome_vfs_volume_get_device_path (volume);
-	char *uri = gnome_vfs_volume_get_activation_uri (volume);
-	char *mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	char *udi = gnome_vfs_volume_get_hal_udi (volume);
+	GFile *file;
+	char *udi, *device_path, *mount_path;
+	gboolean result;
+	GMount *mount;
 
-	gboolean result = gvm_check_dvd_only (udi, device_path, mount_path);
+	if (!self->volume)
+		return FALSE;
+
+	mount = g_volume_get_mount (self->volume);
+	if (!mount)
+	    return FALSE;
+
+	file = g_mount_get_root (mount);
+	g_object_unref (mount);
+	
+	if (!file)
+		return FALSE;
+
+	mount_path = g_file_get_path (file);
+
+	g_object_unref (file);
+
+	device_path = g_volume_get_identifier (self->volume,
+					       G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	udi = g_volume_get_identifier (self->volume,
+				       G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+
+	result = gvm_check_dvd_only (udi, device_path, mount_path);
 
 	g_free (device_path);
 	g_free (udi);
-	g_free (uri);
 	g_free (mount_path);
 
 	return result;
 }
 
 static gboolean
-check_audio_cd (DriveButton *self, GnomeVFSVolume *volume)
+check_audio_cd (DriveButton *self)
 {
-	char *activation_uri = gnome_vfs_volume_get_activation_uri (volume);
-	// we have an audioCD if the activation URI starts by 'cdda://'
+	GFile *file;
+	char *activation_uri;
+	GMount *mount;
+
+	if (!self->volume)
+		return FALSE;
+
+	mount = g_volume_get_mount (self->volume);
+	if (!mount)
+	    return FALSE;
+
+	file = g_mount_get_root (mount);
+	g_object_unref (mount);
+
+	if (!file)
+		return FALSE;
+
+	activation_uri = g_file_get_uri (file);
+
+	g_object_unref (file);
+
+	/* we have an audioCD if the activation URI starts by 'cdda://' */
 	gboolean result = (strncmp ("cdda://", activation_uri, 7) == 0);
 	g_free (activation_uri);
 	return result;
@@ -731,24 +715,30 @@ check_audio_cd (DriveButton *self, GnomeVFSVolume *volume)
 static void
 run_command (DriveButton *self, const char *command)
 {
-	char *uri, *mount_path;
-	char *device_path = gnome_vfs_drive_get_device_path (self->drive);
+	GFile *file;
+	char *mount_path, *device_path;
+	GMount *mount;
 
-	GList *volumes;
-	GnomeVFSVolume *volume;
+	if (!self->volume)
+		return;
 
-	volumes = gnome_vfs_drive_get_mounted_volumes (self->drive);
-	volume = GNOME_VFS_VOLUME (volumes->data);
-	uri = gnome_vfs_volume_get_activation_uri (volume);
-	mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
+	mount = g_volume_get_mount (self->volume);
+	if (!mount)
+	    return;
 
-	gnome_vfs_drive_get_display_name (self->drive);
-	gvm_run_command (device_path, command,
-				 mount_path);
+	file = g_mount_get_root (mount);
+	g_object_unref (mount);
 
-	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_unref, NULL);
-	g_list_free (volumes);
+	g_assert (file);
+
+	mount_path = g_file_get_path (file);
+
+	g_object_unref (file);
+
+	device_path = g_volume_get_identifier (self->volume,
+					       G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+
+	gvm_run_command (device_path, command, mount_path);
 
 	g_free (mount_path);
 	g_free (device_path);
@@ -757,9 +747,11 @@ run_command (DriveButton *self, const char *command)
 static void
 mount_drive (DriveButton *self, GtkWidget *item)
 {
-    if (self->drive) {
-	gnome_vfs_drive_mount (self->drive, mount_result,
-			       GINT_TO_POINTER(CMD_MOUNT_OR_PLAY));
+    if (self->volume) {
+        GMountOperation *mount_op = gtk_mount_operation_new (NULL);
+	g_volume_mount (self->volume, G_MOUNT_MOUNT_NONE,
+			mount_op, NULL, NULL, NULL);
+        g_object_unref (mount_op);
     } else {
 	g_return_if_reached();
     }
@@ -767,25 +759,40 @@ mount_drive (DriveButton *self, GtkWidget *item)
 static void
 unmount_drive (DriveButton *self, GtkWidget *item)
 {
-    if (self->drive) {
-	gnome_vfs_drive_unmount (self->drive, mount_result,
-				 GINT_TO_POINTER(CMD_UNMOUNT));
-    } else if (self->volume) {
-	gnome_vfs_volume_unmount (self->volume, mount_result,
-				  GINT_TO_POINTER(CMD_UNMOUNT));
+    if (self->volume) {
+	GMount *mount;
+
+	mount = g_volume_get_mount (self->volume);
+	if (mount)
+	{
+	    g_mount_unmount (mount, G_MOUNT_UNMOUNT_NONE,
+			     NULL, NULL, NULL);
+	    g_object_unref (mount);
+	}
+    } else if (self->mount) {
+	g_mount_unmount (self->mount, G_MOUNT_UNMOUNT_NONE,
+			 NULL, NULL, NULL);
     } else {
 	g_return_if_reached();
     }
 }
+
+static void eject_finish (DriveButton *self, GAsyncResult *res,
+			  gpointer user_data)
+{
+    /* Do nothing. We shouldn't need this according to the GIO
+     * docs, but the applet crashes without it using glib 2.18.0 */
+}
+
 static void
 eject_drive (DriveButton *self, GtkWidget *item)
 {
-    if (self->drive) {
-	gnome_vfs_drive_eject (self->drive, mount_result,
-			       GINT_TO_POINTER(CMD_EJECT));
-    } else if (self->volume) {
-	gnome_vfs_volume_eject (self->volume, mount_result,
-				GINT_TO_POINTER(CMD_EJECT));
+    if (self->volume) {
+	g_volume_eject (self->volume, G_MOUNT_UNMOUNT_NONE,
+			NULL,(GAsyncReadyCallback) eject_finish, NULL);
+    } else if (self->mount) {
+	g_mount_eject (self->mount, G_MOUNT_UNMOUNT_NONE,
+		       NULL, (GAsyncReadyCallback) eject_finish, NULL);
     } else {
 	g_return_if_reached();
     }
@@ -816,73 +823,29 @@ play_cda (DriveButton *self, GtkWidget *item)
 static void
 drive_button_ensure_popup (DriveButton *self)
 {
-    GnomeVFSDeviceType device_type;
     char *display_name, *tmp, *label;
-    int action = CMD_NONE;
     GtkWidget *item;
-    GCallback callback;
-    const char *action_icon;
-    gboolean ejectable;
+    gboolean mounted, ejectable;
 
     if (self->popup_menu) return;
 
-    if (self->drive) {
-	GnomeVFSVolume *volume = NULL;
-	GList *volumes;
+    mounted = FALSE;
 
-	if (!gnome_vfs_drive_is_connected (self->drive)) return;
+    if (self->volume) {
+	GMount *mount = NULL;
 
-	device_type = gnome_vfs_drive_get_device_type (self->drive);
-	display_name = gnome_vfs_drive_get_display_name (self->drive);
-	ejectable = gnome_vfs_drive_needs_eject (self->drive);
+	display_name = g_volume_get_name (self->volume);
+	ejectable = g_volume_can_eject (self->volume);
 
-	if (gnome_vfs_drive_is_mounted (self->drive)) {
-	    if (!ejectable)
-		action = CMD_UNMOUNT;
-	} else {
-	    action = CMD_MOUNT_OR_PLAY;
+	mount = g_volume_get_mount (self->volume);
+	if (mount) {
+	    mounted = TRUE;
+	    g_object_unref (mount);
 	}
-    
-	volumes = gnome_vfs_drive_get_mounted_volumes (self->drive);
-	if (volumes != NULL)
-	{
-		volume = GNOME_VFS_VOLUME (volumes->data);
-		device_type = gnome_vfs_volume_get_device_type (volume);
-	}
-
-	/*
-	 * For some reason, on my computer, I get GNOME_VFS_DEVICE_TYPE_CDROM
-	 * also for DVDs and Audio CDs, while for DVD the icon is correctly the
-	 * one for DVDs and for Audio CDs the description correctly that of an
-	 * audio CD? So i have this hack.
-	 */
-	if (volume)
-	{
-		if (check_dvd_video (volume))
-		{
-			device_type = GNOME_VFS_DEVICE_TYPE_VIDEO_DVD;
-		}
-		if (check_audio_cd (self, volume))
-		{
-			device_type = GNOME_VFS_DEVICE_TYPE_AUDIO_CD;
-		}
-	}
-
-	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_unref, NULL);
-	g_list_free (volumes);
     } else {
-	GnomeVFSDrive* drive = gnome_vfs_volume_get_drive (self->volume);
-
-	if (!gnome_vfs_volume_is_mounted (self->volume)) return;
-
-	device_type = gnome_vfs_volume_get_device_type (self->volume);
-	display_name = gnome_vfs_volume_get_display_name (self->volume);
-	if (drive)
-	    ejectable = gnome_vfs_drive_needs_eject (drive);
-	else
-	    ejectable = FALSE;
-	if (!ejectable)
-	    action = CMD_UNMOUNT;
+	display_name = g_mount_get_name (self->mount);
+	ejectable = g_mount_can_eject (self->mount);
+	mounted = TRUE;
     }
 
     self->popup_menu = gtk_menu_new ();
@@ -892,57 +855,43 @@ drive_button_ensure_popup (DriveButton *self)
     g_free (display_name);
     display_name = tmp;
 
-	callback = G_CALLBACK (open_drive);
-	action_icon = GTK_STOCK_OPEN;
-
-	switch (device_type) {
-	case GNOME_VFS_DEVICE_TYPE_VIDEO_DVD:
-		label = g_strdup (_("_Play DVD"));
-		callback = G_CALLBACK (play_dvd);
-		action_icon = GTK_STOCK_MEDIA_PLAY;
-		break;
-	case GNOME_VFS_DEVICE_TYPE_AUDIO_CD:
-		label = g_strdup (_("_Play CD"));
-		callback = G_CALLBACK (play_cda);
-		action_icon = GTK_STOCK_MEDIA_PLAY;
-		break;
-	default:
-		label = g_strdup_printf (_("_Open %s"), display_name);
-	}
-
-	item = create_menu_item (self, action_icon, label,
-			     callback,
-			     action != CMD_MOUNT_OR_PLAY);
-    g_free (label);
+    if (check_dvd_video (self)) {
+	item = create_menu_item (self, "media-playback-start",
+				 _("_Play DVD"), G_CALLBACK (play_dvd),
+				 TRUE);
+    } else if (check_audio_cd (self)) {
+	item = create_menu_item (self, "media-playback-start",
+				 _("_Play CD"), G_CALLBACK (play_cda),
+				 TRUE);
+    } else {
+	label = g_strdup_printf (_("_Open %s"), display_name);
+	item = create_menu_item (self, "document-open", label,
+			     G_CALLBACK (open_drive), mounted);
+	g_free (label);
+    }
     gtk_container_add (GTK_CONTAINER (self->popup_menu), item);
 
-    switch (action) {
-    case CMD_MOUNT_OR_PLAY:
+    if (mounted) {
+	if (!ejectable) {
+	    label = g_strdup_printf (_("Un_mount %s"), display_name);
+	    item = create_menu_item (self, NULL, label,
+				     G_CALLBACK (unmount_drive), TRUE);
+	    g_free (label);
+	    gtk_container_add (GTK_CONTAINER (self->popup_menu), item);
+	}
+    } else {
 	label = g_strdup_printf (_("_Mount %s"), display_name);
 	item = create_menu_item (self, NULL, label,
 				 G_CALLBACK (mount_drive), TRUE);
 	g_free (label);
 	gtk_container_add (GTK_CONTAINER (self->popup_menu), item);
-	break;
-    case CMD_UNMOUNT:
-	label = g_strdup_printf (_("Un_mount %s"), display_name);
-	item = create_menu_item (self, NULL, label,
-				 G_CALLBACK (unmount_drive), TRUE);
-	g_free (label);
-	gtk_container_add (GTK_CONTAINER (self->popup_menu), item);
-	break;
-    default:
-	break;
     }
 
-    if (ejectable)
-    {
+    if (ejectable) {
 	label = g_strdup_printf (_("_Eject %s"), display_name);
-	item = create_menu_item (self, NULL, label,
+	item = create_menu_item (self, "media-eject", label,
 				 G_CALLBACK (eject_drive), TRUE);
 	g_free (label);
 	gtk_container_add (GTK_CONTAINER (self->popup_menu), item);
     }
-
-    g_free (display_name);
 }
